@@ -57,8 +57,9 @@ def to_stream(stream, names = None, delimit = b',', line_terminator = b'\n', buf
         
         if names is not None:        
             stream_.write((_stringify_tup(names) if isinstance(names, tuple) else names) + line_terminator)
+            stream_.flush()
 
-        writer = dagpype_c.line_writer(stream_, line_terminator)        
+        writer = dagpype_c.line_writer(stream_.fileno(), line_terminator)        
         
         stored = []
         try:
