@@ -12,10 +12,6 @@
 
 using namespace std;
 
-#if IS_PY3K
-typedef int (*cmpfunc)(PyObject *, PyObject *);
-#endif // #if IS_PY3K
-
 struct ArrayColReader
 {
     PyObject_HEAD
@@ -364,7 +360,7 @@ array_col_reader_iternext(ArrayColReader * self)
     }
 
     // TRACE("Returning " << i);
-    return PyInt_FromLong(i);
+    return pyint_from_long(i);
 }
 
 extern "C" int
@@ -391,17 +387,17 @@ PyTypeObject ArrayColReaderType = {
     0,                                      /*tp_itemsize*/
     /* methods */
     (destructor)array_col_reader_dealloc,          /*tp_dealloc*/
-    (printfunc)0,                           /*tp_print*/
-    (getattrfunc)0,                         /*tp_getattr*/
-    (setattrfunc)0,                         /*tp_setattr*/
-    (cmpfunc)0,                             /*tp_compare*/
-    (reprfunc)0,                            /*tp_repr*/
+    0,                           /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                             /*tp_compare*/
+    0,                            /*tp_repr*/
     0,                                      /*tp_as_number*/
     0,                                      /*tp_as_sequence*/
     0,                                      /*tp_as_mapping*/
-    (hashfunc)0,                            /*tp_hash*/
-    (ternaryfunc)0,                         /*tp_call*/
-    (reprfunc)0,                            /*tp_str*/
+    0,                            /*tp_hash*/
+    0,                         /*tp_call*/
+    0,                            /*tp_str*/
     0,                                      /*tp_getattro*/
     0,                                      /*tp_setattro*/
     0,                                      /*tp_as_buffer*/
