@@ -10,10 +10,6 @@
 
 using namespace std;
 
-#if IS_PY3K
-typedef int (*cmpfunc)(PyObject *, PyObject *);
-#endif // #if IS_PY3K
-
 struct ColReader
 {
     PyObject_HEAD
@@ -260,17 +256,17 @@ PyTypeObject ColReaderType = {
     0,                                      /*tp_itemsize*/
     /* methods */
     (destructor)col_reader_dealloc,          /*tp_dealloc*/
-    (printfunc)0,                           /*tp_print*/
-    (getattrfunc)0,                         /*tp_getattr*/
-    (setattrfunc)0,                         /*tp_setattr*/
-    (cmpfunc)0,                             /*tp_compare*/
-    (reprfunc)0,                            /*tp_repr*/
+    0,                           /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                             /*tp_compare*/
+    0,                            /*tp_repr*/
     0,                                      /*tp_as_number*/
     0,                                      /*tp_as_sequence*/
     0,                                      /*tp_as_mapping*/
-    (hashfunc)0,                            /*tp_hash*/
-    (ternaryfunc)0,                         /*tp_call*/
-    (reprfunc)0,                            /*tp_str*/
+    0,                            /*tp_hash*/
+    0,                         /*tp_call*/
+    0,                            /*tp_str*/
     0,                                      /*tp_getattro*/
     0,                                      /*tp_setattro*/
     0,                                      /*tp_as_buffer*/
