@@ -2,9 +2,16 @@
 #define PARSER_DEFS_HPP
 
 #include <utility>
+#include <vector>
 
 #include "defs.hpp"
 
+#if defined(__GNUC__) && !defined(DAGPYPE_NO_USE_AIO)
+#define DAGPYPE_USE_AIO
+#endif // #if defined(__GNUC__) && !defined(DAGPYPE_NO_USE_AIO)
+
+typedef std::vector<char> buf_t;
+
 enum{_int = 0, _float = 1, _str = 2};
 enum{_inds_cols = 0, _names_cols = 1, _all_cols = 2};
 enum{max_num_cols = 1000, max_field_len = 128};
