@@ -6,10 +6,8 @@ import numpy
 import random
 import collections
 import re
-if sys.version_info >= (3, 0):
-    import io as str_io
-else:
-    import StringIO as str_io
+import itertools
+import string
 
 sys.path.extend(['..', '../dagpype'])
 from dagpype import *
@@ -35,7 +33,7 @@ def _wind_rain_tags():
     return _act
 
 
-class _TestSeq01Basic(unittest.TestCase):
+class _TestSeq00Basic(unittest.TestCase):
     def test_00(self):
         source(open('data/data.csv', 'r'))
 
@@ -115,12 +113,10 @@ class _TestSeq01Streams(unittest.TestCase):
     def test_02(self):
         if os.path.exists('tmp_data.csv'):
             os.remove('tmp_data.csv')
-        n = stream_lines(open('data/data.csv', 'r')) | \
-            to_stream(open('tmp_data.csv', 'w')) 
+        n = stream_lines(open('data/data.csv', 'r')) | to_stream(open('tmp_data.csv', 'w')) 
         self.assertTrue(os.path.exists('tmp_data.csv'))
         self.assertEqual(n, _TestSeq01Streams._num_lines)
-        l = stream_lines(open('tmp_data.csv', 'r')) | \
-            to_list() 
+        l = stream_lines(open('tmp_data.csv', 'r')) | to_list() 
         self.assertEqual(len(l), _TestSeq01Streams._num_lines)
 
     def test_03(self):
@@ -138,6 +134,21 @@ class _TestSeq01Streams(unittest.TestCase):
         n = stream_lines('data/data.csv', False) | count()
         self.assertEqual(n, _TestSeq01Streams._num_lines)
 
+    def test_05(self):
+        for i in range(15):
+            buf_len = max(10, int(random.expovariate(1. / 60)))   
+            stream_len = max(10, int(random.expovariate(1. / 10000)))  
+            rand_line = lambda len_: ''.join(random.choice(string.letters) for _ in range(len_))
+            data = [rand_line(1 + min(int(random.expovariate(1. / 20)), buf_len - 2)) for _ in range(stream_len)]
+            open('long_tmp.txt', 'w').write('\n'.join(data))
+            source(data) | to_stream('long_tmp1.txt', buf_size = buf_len)
+            open('buf_size.txt', 'w').write(str(buf_len))
+            data1 = stream_lines('long_tmp1.txt') | to_list()
+            stream_lines('long_tmp1.txt') | to_stream('long_tmp2.txt', buf_size = buf_len)
+            data2 = stream_lines('long_tmp2.txt') | to_list()
+            self.assertEqual(data, data1)
+            self.assertEqual(data, data2)
+
 
 class _TestSeq02CSV(unittest.TestCase):
     @classmethod
@@ -265,14 +276,6 @@ class _TestSeq02CSV(unittest.TestCase):
         self.assertEqual(n, 34)
 
     def test_17(self):
-        s = str_io.StringIO()
-        n = source([(1, 2), (3, 4)]) | to_stream(s, delimit = '\t')
-        self.assertEqual(n, 2)
-        ls = s.getvalue().split('\n')
-        self.assertGreater(len(ls), 2)
-        self.assertEqual(ls[0].strip(), '1\t2'.strip())
-
-    def test_18(self):
         v = stream_vals(open('data/data.csv', 'r'), ('day', 'wind'), (int, float)) | \
             trace() | \
             nth(1)
@@ -280,7 +283,7 @@ class _TestSeq02CSV(unittest.TestCase):
         self.assertTrue(isinstance(v[1], float))
         self.assertEqual(v, (1, 4))
 
-    def test_19(self):
+    def test_18(self):
         n = stream_vals((' day |wind ',), delimit = '|', skip_init_space = False, types_ = (str, str)) | \
             nth(0)
         self.assertGreater(n, ' day ')
@@ -288,26 +291,26 @@ class _TestSeq02CSV(unittest.TestCase):
             nth(0)
         self.assertGreater(n, 'day ')
 
-    def test_20(self):
+    def test_19(self):
         n = stream_vals((' day |wind ',), delimit = '|', skip_init_space = False, types_ = (str)) | \
             nth(0)
 
-    def test_21(self):
+    def test_20(self):
         with self.assertRaises(InvalidParamError):
             stream_vals('data/data.csv', ('wind', 'fooby'), str, delimit = 'foo') | \
                 to_stream('tmp_data.csv', 'fooby')
 
-    def test_22(self):
+    def test_21(self):
         with self.assertRaises(InvalidParamError):
             stream_vals('data/data.csv', ('wind', 'fooby'), str, comment = 'foo') | \
                 to_stream('tmp_data.csv', 'fooby')
 
-    def test_23(self):
+    def test_22(self):
         with self.assertRaises(InvalidParamError):
             stream_vals('data/data.csv', ('wind', 1), str, comment = 'foo') | \
                 to_stream('tmp_data.csv', 'fooby')
 
-    def test_24(self):
+    def test_23(self):
         l = stream_vals(('1|2', '0|4'), delimit = '|', types_ = (bool, bool)) | nth(-1)
         self.assertEqual(l, (True, True))
         l = stream_vals(('1|2', '0'), delimit = '|', types_ = (bool, bool)) | nth(-1)
@@ -316,14 +319,14 @@ class _TestSeq02CSV(unittest.TestCase):
         self.assertEqual(l, True)
 
     @unittest.expectedFailure
-    def test_25(self):
+    def test_24(self):
         stream_vals('data/bad_data.csv', 'day', int) | sum_()
     
     @unittest.expectedFailure
-    def test_26(self):
+    def test_25(self):
         stream_vals('data/bad_data.csv', 'hail', int) | sum_()
 
-    def test_27(self):
+    def test_26(self):
         l = stream_vals('data/quirky_data.csv', 'day', int) | to_list()
         self.assertEqual(l, [1, 1, None, 4])
         l = stream_vals('data/quirky_data.csv', 'day') | to_list()
@@ -331,7 +334,7 @@ class _TestSeq02CSV(unittest.TestCase):
         l = stream_vals('data/quirky_data.csv', 'day', float) | to_list()
         self.assertEqual(l, [1.0, 1.0, None, 4.0])
 
-    def test_28(self):
+    def test_27(self):
         l = stream_vals('data/neat_quirky_data.csv') | to_list()
         self.assertEqual(l, [(1.0, None, None, 4.0), (1.0, 4.0, None), (None, None, None, None), (4.0, 3.0, 2.0, 1.0)])
         l = stream_vals('data/neat_quirky_data.csv', types_ = (float, int, float, int)) | to_list()
@@ -344,6 +347,13 @@ class _TestSeq02CSV(unittest.TestCase):
         self.assertEqual(
             l, 
             [('1', None, None, 4, None, None), ('1', '4', None, None, None, None), (None, None, None, None, None, None), ('4', '3', 2.0, 1, None, None)])
+   
+    def test_28(self):
+        with open('tmp.tsv', 'wb') as f:
+            for i in range(10):
+                f.write('\t'.join([str(random.random()) for i in range(100)]) + '\n')
+        c = stream_vals('tmp.tsv', delimit = '\t') | count()
+        self.assertEqual(c, 10)
 
 
 class _TestSeq03Control(unittest.TestCase):
@@ -427,7 +437,7 @@ class _TestSeq03Control(unittest.TestCase):
         self.assertEqual(f, ('1', '2', 3))
 
     def test_17(self):
-        f = source([(1, 2, 3, 5), (1, 2, 3, 6)]) | trace() | cast((str, str, float, int)) | to_list()
+        f = source([(1, 2, 3, 5), (1, 2, 3, 6)]) | cast((str, str, float, int)) | to_list()
         self.assertEqual(f, [('1', '2', 3, 5), ('1', '2', 3, 6)])
 
     def test_18(self):
@@ -446,6 +456,100 @@ class _TestSeq03Control(unittest.TestCase):
             tot[j] /= float(n)
             self.assertAlmostEqual(tot[j] / (99. / 2), 1, 1)
             
+    def test_20(self):
+        l = source([1, 2, 3, 4, 3, 2, 1]) | from_(2) | to_list()
+        self.assertEqual(l, [2, 3, 4, 3, 2, 1])
+
+        l = source([1, 2, 3, 4, 3, 2, 1]) | from_(2, False) | to_list()
+        self.assertEqual(l, [3, 4, 3, 2, 1])
+
+        l = source([1, 2, 3, 4, 3, 2, 1]) | from_(lambda d: d % 3 == 0) | to_list()
+        self.assertEqual(l, [3, 4, 3, 2, 1])
+
+    def test_21(self):
+        l = source([1, 2, 3, 4, 3, 2, 1]) | to(2) | to_list()
+        self.assertEqual(l, [1, 2])        
+       
+        l = source([1, 2, 3, 4, 3, 2, 1]) | to(2, False) | to_list()
+        self.assertEqual(l, [1])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1]) | to(lambda d: d % 3 == 0) | to_list()
+        self.assertEqual(l, [1, 2, 3])
+
+    def test_22(self):
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3) | to_list()
+        self.assertEqual(l, [2, 3, 2, 1, 3, 2])
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3, from_inclusive = False) | to_list()
+        self.assertEqual(l, [3, 1, 3])
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3, to_inclusive = False) | to_list()
+        self.assertEqual(l, [2, 2, 1, 2])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | \
+            from_to(2, 3, from_inclusive = False, to_inclusive = False) | to_list()
+        self.assertEqual(l, [1])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3, strict = True) | to_list()
+        self.assertEqual(l, [2, 3, 2, 1, 3])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3, from_inclusive = False, strict = True) | to_list()
+        self.assertEqual(l, [3, 1, 3])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(2, 3, to_inclusive = False, strict = True) | to_list()
+        self.assertEqual(l, [2, 2, 1])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | \
+            from_to(2, 3, from_inclusive = False, to_inclusive = False, strict = True) | to_list()
+        self.assertEqual(l, [1])
+            
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(lambda d: d % 2 == 0, lambda d: d % 4 == 0) | to_list()
+        self.assertEqual(l, [2, 3, 4, 2, 1, 3, 7, 2])        
+
+        l = source([1, 2, 3, 4, 3, 2, 1, 3, 7, 2]) | from_to(lambda d: d % 2 == 0, lambda d: d % 4 == 0, strict = True) | to_list()
+        self.assertEqual(l, [2, 3, 4])        
+        
+    def test_23(self):
+        l = source(range(100)) | slice_(10) | to_list()
+        self.assertEqual(l, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9])        
+
+        l = source(range(100)) | slice_(0, 100, 10) | to_list()
+        self.assertEqual(l, [0, 10, 20, 30, 40, 50, 60, 70, 80, 90])        
+
+        l = source(range(100)) | slice_(0, 10) | to_list()
+        self.assertEqual(l, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9])            
+
+        l = source(range(20)) | slice_(3, None, 7) | to_list()
+        self.assertEqual(l, [3, 10, 17])            
+
+        l = source(range(20)) | slice_(None, None, 7) | to_list()
+        self.assertEqual(l, [0, 7, 14])            
+
+        l = source(range(5)) | slice_(3) | to_list()
+        self.assertEqual(l, [0, 1, 2])            
+
+    @unittest.expectedFailure
+    def test_24(self):
+        source(range(5)) | slice_() | to_list()
+
+    def test_25(self):
+        l = source(range(10)) | tail(4) | to_list()
+        self.assertEqual(l, [6, 7, 8, 9])
+
+        l = source(range(10)) | tail(0) | to_list()
+        self.assertEqual(l, [])
+
+        l = source(range(2)) | tail(4) | to_list()
+        self.assertEqual(l, [0, 1])
+
+    def test_26(self):
+        self.assertEqual(source(['1', '2', '3', '4']) | sum_(), '1234')
+
+    def test_27(self):
+        self.assertEqual(source(['a', 'b', 'c', 'd']) | enumerate_() | to_list(), [(0, 'a'), (1, 'b'), (2, 'c'), (3, 'd')])
+        self.assertEqual(source(['a', 'b', 'c', 'd']) | enumerate_(1) | to_list(), [(1, 'a'), (2, 'b'), (3, 'c'), (4, 'd')])
+        self.assertEqual(source(['a', 'b', 'c', 'd']) | enumerate_(-1) | to_list(), [(-1, 'a'), (0, 'b'), (1, 'c'), (2, 'd')])
+
 
 class _TestSeq04Xml(unittest.TestCase):
     def test_01(self):
@@ -936,6 +1040,26 @@ class _Test11Numpy(unittest.TestCase):
     def test_30(self):
         np.chunk_stream_vals('data/data.csv', ('day', 'wind'), (float, float), ('moshe', 0)) | np.corr()
 
+    def test_31(self):
+        a, b = source((random.random() for _ in xrange(99999))) | \
+            (np.chunk() | np.cum_sum() | np.concatenate_chunks()) + (cum_sum() | np.to_array())
+        self.assertTrue(numpy.allclose(a, b))
+
+    def test_32(self):
+        for i in range(5):
+            chunk_len = max(10, random.expovariate(60))   
+            stream_len = max(10, random.expovariate(1000))     
+            npc, c = source((i, random.random()) for i in range(stream_len)) | \
+                (np.chunk(chunk_len) | np.corr()) + corr()
+        self.assertAlmostEqual(npc, c)
+        
+    def test_33(self):
+        l, r = source([random.random() for i in range(3000)]) | \
+            (np.chunk(max_elems = 99) | np.enumerate_() | (select_inds(0) | np.unchunk()) + (select_inds(1) | np.unchunk())) + \
+                enumerate_() | \
+            nth(-1)
+        self.assertEqual(l, r)                        
+
 
 class _Test12MovingAggs(unittest.TestCase):
     def test_00(self):
@@ -973,6 +1097,10 @@ class _Test12MovingAggs(unittest.TestCase):
         l = source([1, 4, 2, 4, 6, 9, 2, 4, 5]) | window_quantile(3, 0.5) | to_list()
         self.assertEqual(l, [1, 4, 2, 4, 4, 6, 6, 4, 4])
     
+    def test_08(self):
+        a = source([1., 2., 3., 4.]) | cum_sum() | to_list()
+        self.assertEqual(a, [1., 3., 6., 10.])
+
 
 class _Test13Plotting(unittest.TestCase):
     _a = numpy.arange(0, 2, 0.3)
@@ -1102,21 +1230,21 @@ class _Test14NumpyCSV(unittest.TestCase):
     def test_07(self):
         self.assertEqual(
             np.chunk_stream_vals('data/data.csv', ('day', 'wind'), (int, float), ('moshe', 0)) | \
-                filt(lambda (x, y): x) | np.unchunk() | to_list(),
+                filt(lambda xy: xy[0]) | np.unchunk() | to_list(),
             stream_vals('data/data.csv', 'day', int) | to_list())
         self.assertEqual(
             np.chunk_stream_vals('data/data.csv', ('day', 'wind'), (int, float), ('moshe', 0)) | \
-                filt(lambda (x, y): y) | np.unchunk() | to_list(),
+                filt(lambda xy: xy[1]) | np.unchunk() | to_list(),
             stream_vals('data/data.csv', 'wind', float) | to_list())
 
     def test_08(self):
         self.assertEqual(
             np.chunk_stream_vals('data/data.csv', ('wind', 'day'), (float, int), ('moshe', 0)) | \
-                filt(lambda (x, y): y) | np.unchunk() | to_list(),
+                filt(lambda xy: xy[1]) | np.unchunk() | to_list(),
             stream_vals('data/data.csv', 'day', int) | to_list())
         self.assertEqual(
             np.chunk_stream_vals('data/data.csv', ('wind', 'day'), (float, int), ('moshe', 0)) | \
-                filt(lambda (x, y): x) | np.unchunk() | to_list(),
+                filt(lambda xy: xy[0]) | np.unchunk() | to_list(),
             stream_vals('data/data.csv', 'wind', float) | to_list())
 
     def test_09(self):
@@ -1143,6 +1271,110 @@ class _TestSeq15filterCSV(unittest.TestCase):
         l = source(['foo,bar,wind', '0,1,2', '2,3.24,4']) | csv_split(cols = 'bar', types_ = float) | to_list()
         self.assertEqual(l, [1, 3.24])
 
+    def test_02(self):
+        l = source(['0,1,2', '2,3.24,4']) | csv_split(cols = 1, types_ = float) | to_list()
+        self.assertEqual(l, [1, 3.24])
+
+
+class _TestSeq16Sed(unittest.TestCase):
+    def test_02(self):
+        stream_lines('data/foo_sed_2.txt') | filt(lambda l: l + '\n', pre = lambda l: l.strip()) | to_stream('tmp2.txt')
+        self.assertEqual(open('tmp2.txt').read(), open('data/bar_sed_2.txt').read())
+        
+    def test_04(self):
+        stream_lines('data/foo_sed_4.txt') | enumerate_() | \
+            consec_group(lambda nl: int(nl[0] / 2), lambda d: select_inds(1) | nth(0))  | \
+            to_stream('tmp4.txt')
+        self.assertEqual(open('tmp4.txt').read(), open('data/bar_sed_4.txt').read())
+
+    def test_10(self):
+        source((int(l.strip() != ''), l.strip()) for l in open('data/foo_sed_10.txt')) | \
+            (select_inds(0) | cum_sum()) + select_inds(1) | \
+            filt(lambda (n, l): '{} {}'.format(n, l) if l else '') | \
+            to_stream('tmp10.txt')
+        self.assertEqual(open('tmp10.txt').read(), open('data/bar_sed_10.txt').read())
+
+    def test_29(self):
+        stream_lines('data/foo_sed_29.txt') | \
+            filt(lambda l: 'foo'.join(l.split('foo', 4)[: 4]) + 'bar' + ''.join(l.split('foo', 4)[4: ]) \
+                if len(l.split('foo')) > 3 else l) | \
+            to_stream('tmp29.txt')
+        self.assertEqual(open('tmp29.txt').read(), open('data/bar_sed_29.txt').read())
+
+    def test_38(self):
+        stream_lines('data/foo_sed_38.txt') | enumerate_() | \
+            consec_group(
+                lambda (n, _): int(n / 2), 
+                lambda d: select_inds(1) | (to_list() | sink(lambda l: '\t'.join(l))))  | \
+            to_stream('tmp38.txt')
+        self.assertEqual(open('tmp38.txt').read(), open('data/bar_sed_38.txt').read())
+
+    def test_39(self):
+        c = [0]
+        stream_lines('data/foo_sed_39.txt') | \
+            consec_group(
+                lambda l: (c[0], c.__setitem__(0, c[0] + int(len(l) == 0 or l[-1] != '\\'))), 
+                lambda d: filt(lambda l: l[: -1] if len(l) > 0 and l[-1] == '\\' else l) | sum_())  | \
+            to_stream('tmp39.txt')
+        self.assertEqual(open('tmp39.txt').read(), open('data/bar_sed_39.txt').read())
+
+    def test_40(self):
+        c = [0]
+        stream_lines('data/foo_sed_40.txt') | \
+            consec_group(
+                lambda l: (c[0], c.__setitem__(0, c[0] + int(len(l) == 0 or l[0] != '='))), 
+                lambda d: filt(lambda l: l[1 :] if len(l) > 0 and l[0] == '=' else l) | sum_())  | \
+            to_stream('tmp40.txt')
+        self.assertEqual(open('tmp40.txt').read(), open('data/bar_sed_40.txt').read())
+
+    def test_43(self):
+        stream_lines('data/foo_sed_43.txt') | enumerate_(1) | \
+            filt(lambda (n, l): l if n % 5 else (l + '\n')) | to_stream('tmp43.txt')
+        self.assertEqual(open('tmp43.txt').read(), open('data/bar_sed_43.txt').read())
+
+    def test_52(self):
+        r = re.compile(r'foo(.+?)bar')
+        stream_lines('data/foo_sed_52.txt') | to(lambda l: r.search(l)) | (nth(-2) | to_stream('tmp52.txt'))
+        self.assertEqual(open('tmp52.txt').read(), open('data/bar_sed_52.txt').read())
+
+    def test_53(self):
+        r = re.compile(r'foo(.+?)bar')
+        stream_lines('data/foo_sed_53.txt') | from_(lambda l: r.search(l)) | (nth(1) | to_stream('tmp53.txt'))
+        self.assertEqual(open('tmp53.txt').read(), open('data/bar_sed_53.txt').read())
+
+    def test_54(self):
+        r = re.compile(r'foo(.+?)bar')
+        l = stream_lines('data/foo_sed_54.txt') | \
+            (to(lambda l: r.search(l)) | (nth(-2))) + \
+            (from_(lambda l: r.search(l)) | (nth(1))) + \
+            (enumerate_() | filt(pre = lambda (n, l): r.search(l)) | nth(0))
+        self.assertEqual(l, ('bfoo', '', (2, 'cfoodbar')))
+
+    def test_58(self):
+        stream_lines('data/foo_sed_58.txt') | \
+            consec_group(
+                lambda l: l.strip() == '',
+                lambda is_para: to_list()) | \
+            filt(
+                lambda ls: '\n'.join(ls) + '\n', 
+                pre = lambda ls: sum(['AAA' in l for l in ls]) > 0) | \
+            to_stream('tmp58.txt')
+        self.assertEqual(open('tmp58.txt').read(), open('data/bar_sed_58.txt').read())
+        
+    def test_66(self):        
+        l = source(range(20)) | slice_(3, None, 7) | to_list()
+        self.assertEqual(l, [3, 10, 17])
+
+    def test_69(self):        
+        l = source([1, 1, 2, 1, 1, 4, 4, 2, 3, 5]) | \
+            consec_group(lambda l: l, lambda l: nth(0)) | to_list()
+        self.assertEqual(l, [1, 2, 1, 4, 2, 3, 5])
+
+    def test_70(self):        
+        l = source([1, 1, 2, 1, 1, 4, 4, 2, 3, 5]) | \
+            group(lambda l: l, lambda l: nth(0)) | to_list()
+        self.assertEqual(l, [1, 2, 4, 3, 5])
+
 
 if __name__ == '__main__':
     unittest.main()
